

print.cem.att <- function(x,full=FALSE,...){
 if(!is.null(x$tab)){
  cat("\n")
  print(x$tab)
  }
 if(!is.null(x$reg.cem)){
  if(x$extrapolate)
   cat(sprintf("\n%s with extra
   polation:\n\n",x$mod.type))
  else
   cat(sprintf("\n%s on CEM matched data:\n\n",x$mod.type))
    
  if(full){
   print(x$reg.cem)
  } else {
   tmp <- x$reg.cem[,x$treatment]
   cat(sprintf("SATT point estimate: %f (p.value=%f)\n95%% conf. interval: [%f, %f]\n\n", 
     tmp["Estimate"], tmp["p-value"], tmp["Estimate"] + tmp["Std. Error"]*qnorm(0.025), 
	 tmp["Estimate"] + tmp["Std. Error"]*qnorm(0.975)))
  }
  }
 if(!is.null(x$reg.all)){
  if(x$extrapolate)
   cat(sprintf("\n%s with extrapolation:\n\n",x$mod.type))
  else
   cat(sprintf("\n%s on CEM matched data:\n\n",x$mod.type))
  if(full){
   print(x$reg.all)
  } else {
   tmp <- x$reg.all[,x$treatment]
   cat(sprintf("SATT point estimate: %f (p.value=%f)\n95%% conf. interval: [%f, %f]\n\n", 
     tmp["Estimate"], tmp["p-value"], tmp["Estimate"] + tmp["Std. Error"]*qnorm(0.025), 
	 tmp["Estimate"] + tmp["Std. Error"]*qnorm(0.975)))
  }
  }  
 if(!is.null(x$random.cem)){
  if(x$extrapolate)
   cat(sprintf("\n%s with extrapolation:\n\n",x$mod.type))
  else
   cat(sprintf("\n%s on CEM matched data:\n\n",x$mod.type))
   if(full){
    print(x$random.cem)
   } else {
    tmp <- x$random.cem[,x$treatment]
    cat(sprintf("SATT point estimate: %f (p.value=%f)\n95%% conf. interval: [%f, %f]\n\n", 
     tmp["Estimate"], tmp["p-value"], tmp["Estimate"] + tmp["Std. Error"]*qnorm(0.025), 
	 tmp["Estimate"] + tmp["Std. Error"]*qnorm(0.975)))
   }
  }

 if(!is.null(x$random.all)){
  if(x$extrapolate)
   cat(sprintf("\n%s with extrapolation:\n\n",x$mod.type))
  else
   cat(sprintf("\n%s on CEM matched data:\n\n",x$mod.type))
   if(full){
    print(x$random.all)
   } else {
    tmp <- x$random.all[,x$treatment]
    cat(sprintf("SATT point estimate: %f (p.value=%f)\n95%% conf. interval: [%f, %f]\n\n", 
     tmp["Estimate"], tmp["p-value"], tmp["Estimate"] + tmp["Std. Error"]*qnorm(0.025), 
	 tmp["Estimate"] + tmp["Std. Error"]*qnorm(0.975)))
   }
  }
  
}

plot.cem.att <- function(x,...){
 if(!is.null(x$cem.model)){
  plot(x$cem.model, main="Fixed effects model on the CEM matched subsample")
  }
  
 if(!is.null(x$ranef.cem)){
    p1 <- plot(x$ranef.cem,main="Random effects model on CEM matched subsample")
    print(p1,new=TRUE)
  }

 if(!is.null(x$ranef.all)){
   p2 <- plot(x$ranef.all, main="Random effects model on the whole sample")
   print(p2,new=TRUE)
  }
  
}


############### THE LME STUFF ##############


`att` <-
function (obj, formula, data, model="linear", extrapolate=FALSE) 
{

 random = FALSE
 mod.type <- NULL
 family <- "binomial"
 if((class(obj)[1] != "cem.match") && (class(obj)[1] != "cem.match.list"))
  stop("Argument `obj' must be a `cem.match' or `cem.match.list' object") 

 mods <- c("linear", "lm", "lme", "linear-RE", "logit", "logit-RE")
 if(!(model %in% mods))
  stop(sprintf("model `%s' not yet implemented",model))

 if(model %in% c("lm", "linear")){
  random <- FALSE
  mod <- "lm"
  mod.type <- "Linear regression model"
 }

 if(model %in% c("lme", "linear-RE")){
  random <- TRUE
  mod <- "lm"
  mod.type <- "Linear random effect model"
 }

 
  random.cem <- NULL
  random.all <- NULL
  ranef.cem <- NULL
  ranef.all <- NULL
  reg.cem <- NULL
  reg.all <- NULL
  
  if(class(obj)[1] == "cem.match"){

  if(mod=="lm"){
   out <- do.call(mod,list(formula=formula, data=data, weights=obj$w))
  
    data1 <- data
    data1$allID <- factor(obj$strata)
    data1$cemID <- factor(obj$mstrata)
	data1$matched <- obj$matched
    f1 <- formula(sprintf(" ~ %s | allID",obj$treatment))   
    f2 <- formula(sprintf(" ~ %s | cemID",obj$treatment))   
    control= lmeControl(maxIter=1000,opt="optim")

   if(random & extrapolate){     
    require(nlme, quietly=TRUE)
  # all data
    rand.try <- try(lme(update(formula, ~ . -cemID -allID - matched), data=data1, 
	  random = f1, keep.data=FALSE, control=control), silent=TRUE)
    if(class(rand.try) == "try-error"){
     cat("\nCannot estimate the random effects model for the complete data set\n")  
	} else { 
#   rand.all <- lme(formula, data=data1, random = f1)
     rand.all <- rand.try
     random.all <- t(summary(rand.all)$tTable)
#     rownames(random.all) <- c("Estimate", "Std. Error", "DF", "t value", "p-value")
     ranef.all <- random.effects(rand.all)
	 
	 rand.cf <- coef(rand.all)
     ww <- table(obj$strata, obj$groups)[,2] # hard coded

     cf.idx <- match(rownames(rand.cf), names(ww))
     rownames(random.all) <- c("Estimate", "Std. Error", "DF", "t value", "p-value")
	 w.coef <- apply(rand.cf, 2, function(x) weighted.mean( x, w=ww[cf.idx]))
     random.all["Estimate", ] <- w.coef
	 random.all["Std. Error",] <-  random.all["Std. Error",]*sqrt(length(ww)/sum(ww))
     random.all["t value",] <-  random.all["Estimate",]/random.all["Std. Error",]
     random.all["p-value",] <-  2*(1-pnorm(random.all["t value",]))
	}
   }
# restricted
  if(random & !extrapolate){
    rand.try <- try(lme(update(formula, ~ . -cemID -allID - matched), data=data1, random = f2, 
	 subset=matched==TRUE, keep.data=FALSE, control=control), silent=TRUE)
    if(class(rand.try) == "try-error"){
     cat("\nCannot estimate the random effects model for the CEM matched subsample\n")   
    } else {
#   rand.cem <- lme(formula, data=data1, random = f2, subset=matched==TRUE)
    rand.cem <- rand.try
	random.cem <- t(summary(rand.cem)$tTable)
	rand.cf <- coef(rand.cem)
    ww <- table(obj$mstrata, obj$groups)[,2] # hard coded
    cf.idx <- match(rownames(rand.cf), names(ww))
    rownames(random.cem) <- c("Estimate", "Std. Error", "DF", "t value", "p-value")
	w.coef <- apply(rand.cf, 2, function(x) weighted.mean( x, w=ww[cf.idx]))
    random.cem["Estimate", ] <- w.coef
	random.cem["Std. Error",] <-  random.cem["Std. Error",]*sqrt(length(ww)/sum(ww))
    random.cem["t value",] <-  random.cem["Estimate",]/random.cem["Std. Error",]
    random.cem["p-value",] <-  2*(1-pnorm(random.cem["t value",]))
    ranef.cem <- random.effects(rand.cem)
    }
   }
   
  if(!random){
   out <- do.call(mod, list(formula=formula, data=data, weights=obj$w))
  
   reg.cem <- t(summary(out)$coefficients)
   rownames(reg.cem) <-  c("Estimate", "Std. Error",  "t value", "p-value")
   if(extrapolate){
    idx2 <- which(!obj$matched & obj$groups==obj$g.names[2])
    f3 <- formula(sprintf(" ~  . -%s ",obj$treatment)) 
    f4 <- update(formula, f3)
    y1 <- predict(out, newdata=data[idx2,])
    response <- all.vars(f4)[attr(terms(formula),"response")]
    tmp <- t.test(y1, data[idx2, response])
    att4 <- diff(tmp$estimate)
    att4.serr <- abs(diff(tmp$estimate)/tmp$statistic)
    p <- obj$tab[2,2]/obj$tab[1,2]
    att4 <-  (1-p) * att4 + p * reg.cem["Estimate",obj$treatment]
    reg.all <- matrix(NA, 4,1)
    rownames(reg.all) <- rownames(reg.cem)
    colnames(reg.all) <- obj$treatment
    reg.all["Estimate",1] <- att4 
    reg.all["Std. Error",1] <- sqrt( p^2 * reg.cem["Std. Error", obj$treatment]^2 +
      (1-p)^2 * att4.serr^2  )
    reg.all["t value",1] <- att4/att4.serr 
    reg.all["p-value",1] <- 2*(1-pnorm(att4/att4.serr))
    reg.cem <- NULL
   } 
  }  
  out <- list(reg.cem = reg.cem, random.cem=random.cem, random.all=random.all, tab=obj$tab, 
   ranef.cem=ranef.cem, ranef.all=ranef.all, cem.model = out, treatment=obj$treatment,
   reg.all = reg.all,   extrapolate=extrapolate, mod.type=mod.type)
  class(out) <- "cem.att"
  return(out)
 }
 }
 if(class(data) != "list")
  stop("Argument `data' must be a list of `data.frame's")

 n.cems <- length(obj) - 1

 if(length(data) != n.cems)
  stop("lengths of `cem.match.list' object and `data' do not match")
  
 est <- vector(n.cems, mode="list")
 for(i in 1:n.cems){ 
  if(mod=="lm")
   out <- do.call(mod, list(formula=formula, data=data[[i]], weights=obj[[i]]$w))
  else
   out <- do.call(mod, list(formula=formula, data=data[[i]], weights=obj[[i]]$w,family=family))
  aa <- summary(out)
  est[[i]] <- aa$coefficients 
 }
 qoi <- numeric(dim(est[[1]])[1])
 seq <- numeric(dim(est[[1]])[1])
 for(i in 1:n.cems){
  qoi <- qoi + est[[i]][,1]
  seq <- seq + est[[i]][,2]^2
 }
 
 qoi <- qoi/n.cems
 seq <- seq/n.cems

 s2 <- numeric(dim(est[[1]])[1])
 for(i in 1:n.cems){
  s2 <- s2 + (est[[i]][,1] - qoi)^2
 }
 s2 <- s2/(n.cems-1)

 S <- sqrt(seq + s2*(1+1/n.cems))

 mat <- rbind(qoi, S)
 rownames(mat) <- c("Estimate", "Std. Error")
 
 out <- list(mult=est, model = mat, treatment=obj$treatment) 
 class(out) <- "cem.att"
 out
}

