\name{att}
\alias{att}
\title{Example of ATT estimation from CEM output}
\description{
An example of ATT estimation from CEM output
}
\usage{
att(obj, formula, data, model="linear", extrapolate=FALSE)
}
\arguments{
  \item{obj}{a \code{cem.atch} or \code{cem.match.list} object}
  \item{formula}{a model formula. See Details.}
  \item{data}{a single data.frame or a list of data.frame's in case of \code{cem.match.list}}
  \item{model}{one model. See Details.}
  \item{extrapolate}{extrapolate the CEM restriced estimate to the whole data. Default = \code{FALSE}.}
}
\details{

Argument \code{model} can be \code{lm, linear} for linear regression
model; \code{logit} for the the logistic model;
\code{lme, linear-RE} for the linear model with random effects. 

If the outcome is \code{y} and the
treatment variable is \code{T}, then a \code{formula} like \code{y ~ T}
will produce the simplest estimate the ATT: with lm, it is just the
coefficient on \code{T}, which is the same as the difference in means,
weighted by CEM stratum size.  Users can add covariates to span any
remaining imbalance after the match, such as \code{y ~ T + age + sex},
to adjust for variables \code{age} and \code{sex}.

In the case of multiply imputed datasets, the model is applied to each
single matched data and the ATT and is the standard error estimated
using the standard formulas for combining results of multiply imputed
data.  

When \code{extrapolate} = \code{TRUE}, the estimate model is extrapolated
to the whole set of data.


There is a \code{print} method for the output of \code{att}. Specifying the
option \code{TRUE} in a \code{print} command gives complete output from the
estimated model. See the examples.

}
\value{
  A matrix of estimates with their standard error, or a list in
  the case of \code{cem.match.list}.  } 
\references{Stefano Iacus, Gary King, Giuseppe Porro, ``Matching for
  Casual Inference Without Balance Checking,''
  http://gking.harvard.edu/files/abs/cem-abs.shtml}
\author{Stefano Iacus, Gary King, and Giuseppe Porro}

\examples{ 
data(LL)

# cem match: automatic bin choice
mat <- cem(treatment="treated",data=LL, drop="re78")
mat
mat$k2k

# ATT estimate
att(mat, re78~treated,  data=LL)
att(mat, re78~treated,  data=LL, extra=TRUE)
att(mat, re78~treated,  data=LL, model="linear-RE")
att(mat, re78~treated,  data=LL, model="linear-RE", extra=TRUE)




# reduce the match into k2k using euclidean distance within cem strata
mat2 <- k2k(mat, LL, "euclidean", 1)
mat2
mat2$k2k

# ATT estimate after k2k
att(mat2, re78~treated, data=LL)

# example with missing data
# using multiply imputated data
# we use Amelia for multiple imputation
if(require(Amelia)){
 data(LL)
 n <- dim(LL)[1]
 k <- dim(LL)[2]

# we generate missing values in 30% of the rows of LL data
# randomly in one colum per row
 LL1 <- LL
 idx <- sample(1:n, .3*n)
 invisible(sapply(idx, function(x) LL1[x,sample(2:k,1)] <<- NA))


 imputed <- amelia(LL1)[1:5] 

 mat <- cem("treated", datalist=imputed, data=LL1, drop="re78")

 print(mat)
  
 att(mat, re78 ~ treated, data=imputed)
}
}
\keyword{multivariate}

